<?php

namespace chillerlan\FastRoute;

use chillerlan\FastRoute\DataGenerator\DataGeneratorInterface;
use chillerlan\FastRoute\RouteParser\RouteParserInterface;

class RouteCollector{

	private $routeParser;
	private $dataGenerator;

	/**
	 * Constructs a route collector.
	 *
	 * @param RouteParserInterface   $routeParser
	 * @param DataGeneratorInterface $dataGenerator
	 */
	public function __construct(RouteParserInterface $routeParser, DataGeneratorInterface $dataGenerator){
		$this->routeParser = $routeParser;
		$this->dataGenerator = $dataGenerator;
	}

	/**
	 * Adds a route to the collection.
	 *
	 * The syntax used in the $route string depends on the used route parser.
	 *
	 * @param string|string[] $httpMethod
	 * @param string          $route
	 * @param mixed           $handler
	 */
	public function addRoute($httpMethod, string $route, $handler){
		$routeDatas = $this->routeParser->parse($route);

		foreach((array)$httpMethod as $method){
			foreach($routeDatas as $routeData){
				$this->dataGenerator->addRoute($method, $routeData, $handler);
			}
		}
	}

	/**
	 * Returns the collected route data, as provided by the data generator.
	 *
	 * @return array
	 */
	public function getData():array{
		return $this->dataGenerator->getData();
	}

}
