<?php
/**
 *
 * @filesource   Router.php
 * @created      20.03.2016
 * @package      chillerlan\FastRoute
 * @author       Smiley <smiley@chillerlan.net>
 * @copyright    2016 Smiley
 * @license      MIT
 */

namespace chillerlan\FastRoute;

use chillerlan\FastRoute\DataGenerator\GeneratorGCB;
use chillerlan\FastRoute\Dispatcher\DispatcherInterface;
use chillerlan\FastRoute\Dispatcher\DispatcherGCB;
use chillerlan\FastRoute\RouteCollector;
use chillerlan\FastRoute\RouteParser\RouteParserStd;

/**
 * Class Router
 */
class Router{

	const OPTIONS = [
		'dataGenerator'  => GeneratorGCB::class,
		'dispatcher'     => DispatcherGCB::class,
		'routeCollector' => RouteCollector::class,
		'routeParser'    => RouteParserStd::class,
	];

	/**
	 * @param callable $routeDefinitionCallback
	 * @param array    $options
	 *
	 * @return DispatcherInterface
	 */
	public static function simpleDispatcher(callable $routeDefinitionCallback, array $options = []):DispatcherInterface{
		$options += self::OPTIONS;

		/** @var RouteCollector $routeCollector */
		$routeCollector = new $options['routeCollector'](new $options['routeParser'], new $options['dataGenerator']);
		$routeDefinitionCallback($routeCollector);

		return new $options['dispatcher']($routeCollector->getData());
	}

	/**
	 * @param callable $routeDefinitionCallback
	 * @param array    $options
	 *
	 * @return DispatcherInterface
	 */
	public static function cachedDispatcher(callable $routeDefinitionCallback, array $options = []):DispatcherInterface{
		$options += self::OPTIONS;
		$options += [
			'cacheDisabled'  => false,
		];

		if(!isset($options['cacheFile'])){
			throw new \LogicException('Must specify "cacheFile" option');
		}

		if(!$options['cacheDisabled'] && file_exists($options['cacheFile'])){
			$dispatchData = require $options['cacheFile'];

			if(!is_array($dispatchData)){
				throw new \RuntimeException('Invalid cache file "'.$options['cacheFile'].'"');
			}

			return new $options['dispatcher']($dispatchData);
		}

		$routeCollector = new $options['routeCollector'](new $options['routeParser'], new $options['dataGenerator']);
		$routeDefinitionCallback($routeCollector);

		/** @var RouteCollector $routeCollector */
		$dispatchData = $routeCollector->getData();
		file_put_contents($options['cacheFile'], '<?php return '.var_export($dispatchData, true).';');

		return new $options['dispatcher']($dispatchData);
	}

}
